@@ -239,7 +239,12 @@ asc_compose_set_prefix (AscCompose *compose, const gchar *prefix)
 {
 	AscComposePrivate *priv = GET_PRIVATE (compose);
 	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->mutex);
-	as_ref_string_assign_safe (&priv->prefix, prefix);
+
+	/* do a bit of sanitizing: "no prefix" means the prefix directory is the root directory */
+	if (prefix == NULL || g_strcmp0 (prefix, "") == 0)
+		as_ref_string_assign_safe (&priv->prefix, "/");
+	else
+		as_ref_string_assign_safe (&priv->prefix, prefix);
 }
 
 /**
@@ -1076,7 +1081,7 @@ asc_compose_process_icons (AscCompose *compose,
 		/* create a remote reference if we have data for it */
 		if (priv->media_result_dir != NULL && icon_state != ASC_ICON_STATE_CACHED_ONLY) {
 			g_autofree gchar *icons_media_urlpart_dir = NULL;
-			g_autofree gchar *icons_media_urlpart_fname = NULL;
+			g_autofree gchar *icon_media_urlpart_fname = NULL;
 			g_autofree gchar *icons_media_path = NULL;
 			g_autofree gchar *icon_media_fname = NULL;
 			g_autoptr(AsIcon) remote_icon = NULL;
@@ -1084,7 +1089,7 @@ asc_compose_process_icons (AscCompose *compose,
 								   asc_result_gcid_for_component (cres, cpt),
 								   "icons",
 								   res_icon_size_str);
-			icons_media_urlpart_fname = g_strdup_printf ("%s/%s",
+			icon_media_urlpart_fname = g_strdup_printf ("%s/%s",
 								     icons_media_urlpart_dir,
 								     res_icon_basename);
 			icons_media_path = g_build_filename (priv->media_result_dir,
@@ -1112,7 +1117,18 @@ asc_compose_process_icons (AscCompose *compose,
 			as_icon_set_width (remote_icon, size);
 			as_icon_set_height (remote_icon, size);
 			as_icon_set_scale (remote_icon, scale_factor);
-			as_icon_set_url (remote_icon, icons_media_urlpart_fname);
+
+			/* We can only make use of the media-baseurl-using partial URLs if screenshot storage
+			 * is also enabled, because otherwise screenshots will use full URLs which conflicts
+			 * with the media baseurl (as it is unconditionally prefixed to *all* media URLs */
+			if (as_flags_contains (priv->flags, ASC_COMPOSE_FLAG_STORE_SCREENSHOTS)) {
+				as_icon_set_url (remote_icon, icon_media_urlpart_fname);
+			} else {
+				g_autofree gchar *icon_remote_url = g_strconcat (priv->media_baseurl, "/", icon_media_urlpart_fname, NULL);
+				/* if priv->media_result_dir is set, media_baseurl will be set too (checked before each run) */
+				as_icon_set_url (remote_icon, icon_remote_url);
+			}
+
 			as_component_add_icon (cpt, remote_icon);
 		}
 
@@ -1350,6 +1366,12 @@ asc_compose_process_task_cb (AscComposeTask *ctask, AscCompose *compose)
 	/* open our unit for reading */
 	if (!asc_unit_open (ctask->unit, &tmp_error)) {
 		g_warning ("Failed to open unit: %s", tmp_error->message);
+		asc_result_add_hint (ctask->result,
+					NULL,
+					"unit-read-error",
+					"name", asc_unit_get_bundle_id (ctask->unit),
+					"msg", tmp_error->message,
+					NULL);
 		return;
 	}
 	contents = asc_unit_get_contents (ctask->unit);
@@ -1360,6 +1382,8 @@ asc_compose_process_task_cb (AscComposeTask *ctask, AscCompose *compose)
 	mi_fnames = g_ptr_array_new_with_free_func (g_free);
 	de_fname_map = g_hash_table_new_full (g_str_hash, g_str_equal,
 					      g_free, g_free);
+
+	g_debug ("Looking for metainfo files in: %s", metainfo_dir);
 	for (guint i = 0; i < contents->len; i++) {
 		const gchar *fname = g_ptr_array_index (contents, i);
 
@@ -1929,7 +1953,7 @@ asc_compose_export_hints_data_html (AscCompose *compose, GError **error)
 }
 
 static gboolean
-asc_compose_save_metadata_result (AscCompose *compose, GError **error)
+asc_compose_save_metadata_result (AscCompose *compose, gboolean *results_not_empty, GError **error)
 {
 	AscComposePrivate *priv = GET_PRIVATE (compose);
 	g_autoptr(AsMetadata) mdata = NULL;
@@ -1939,7 +1963,10 @@ asc_compose_save_metadata_result (AscCompose *compose, GError **error)
 	mdata = as_metadata_new ();
 	as_metadata_set_format_style (mdata, AS_FORMAT_STYLE_COLLECTION);
 	as_metadata_set_format_version (mdata, AS_FORMAT_VERSION_CURRENT);
-	if (priv->media_baseurl != NULL)
+
+	/* Set baseurl only if one is set and we actually store any screenshot media. If no screenshot media
+	 * is stored, upstream's URLs are used and having a media base URL makes no sense */
+	if (priv->media_baseurl != NULL && as_flags_contains (priv->flags, ASC_COMPOSE_FLAG_STORE_SCREENSHOTS))
 		as_metadata_set_media_baseurl (mdata, priv->media_baseurl);
 
 	if (priv->format == AS_FORMAT_KIND_YAML)
@@ -1957,14 +1984,19 @@ asc_compose_save_metadata_result (AscCompose *compose, GError **error)
 		return FALSE;
 	}
 
+	if (results_not_empty != NULL)
+		*results_not_empty = FALSE;
+
 	for (guint i = 0; i < priv->results->len; i++) {
 		g_autoptr(GPtrArray) cpts = NULL;
 		AscResult *result = ASC_RESULT (g_ptr_array_index (priv->results, i));
 		cpts = asc_result_fetch_components (result);
-		for (guint j = 0; j < cpts->len; j++) {
+		for (guint j = 0; j < cpts->len; j++)
 			as_metadata_add_component (mdata,
 						   AS_COMPONENT(g_ptr_array_index (cpts, j)));
-		}
+
+		if (cpts->len > 0 && results_not_empty != NULL)
+			*results_not_empty = TRUE;
 	}
 
 	data_fname = g_build_filename (priv->data_result_dir, data_basename, NULL);
@@ -2021,6 +2053,7 @@ asc_compose_run (AscCompose *compose, GCancellable *cancellable, GError **error)
 	AscComposePrivate *priv = GET_PRIVATE (compose);
 	g_autoptr(GPtrArray) tasks = NULL;
 	gboolean temp_dir_created = FALSE;
+	gboolean results_generated = FALSE;
 
 	/* ensure icon output dir is set, hint and data output dirs are optional */
 	if (priv->icons_result_dir == NULL && !as_flags_contains (priv->flags, ASC_COMPOSE_FLAG_IGNORE_ICONS)) {
@@ -2111,16 +2144,6 @@ asc_compose_run (AscCompose *compose, GCancellable *cancellable, GError **error)
 						     compose);
 	}
 
-	if (g_hash_table_size (priv->allowed_cids) > 0 &&
-	    priv->results->len == 0 &&
-	    tasks->len > 0) {
-		/* we had filters set but generated no results - this was most certainly not intended */
-		AscComposeTask *ctask = g_ptr_array_index (tasks, 0);
-		asc_result_add_hint_simple (ctask->result,
-						NULL,
-						"filters-but-no-output");
-	}
-
 	/* collect results */
 	for (guint i = 0; i < tasks->len; i++) {
 		AscComposeTask *ctask = g_ptr_array_index (tasks, i);
@@ -2129,10 +2152,19 @@ asc_compose_run (AscCompose *compose, GCancellable *cancellable, GError **error)
 
 	/* write result */
 	if (priv->data_result_dir != NULL) {
-		if (!asc_compose_save_metadata_result (compose, error))
+		if (!asc_compose_save_metadata_result (compose, &results_generated, error))
 			return NULL;
 	}
 
+	/* check if we (unexpectedly) had no results */
+	if (g_hash_table_size (priv->allowed_cids) > 0 && !results_generated) {
+		/* we had filters set but generated no results - this was most certainly not intended */
+		AscComposeTask *ctask = g_ptr_array_index (tasks, 0);
+		asc_result_add_hint_simple (ctask->result,
+						NULL,
+						"filters-but-no-output");
+	}
+
 	/* write hints */
 	if (priv->hints_result_dir != NULL) {
 		if (!asc_compose_export_hints_data_yaml (compose, error))
